{-# LANGUAGE TypeApplications #-}


-- | Copyright  : Will Thompson and Iñaki García Etxebarria
-- License    : LGPL-2.1
-- Maintainer : Iñaki García Etxebarria
-- 
-- A t'GI.GObject.Objects.ParamSpec.ParamSpec' derived structure that contains the meta data for t'GI.GObject.Structs.ValueArray.ValueArray' properties.

#if (MIN_VERSION_haskell_gi_overloading(1,0,0) && !defined(__HADDOCK_VERSION__))
#define ENABLE_OVERLOADING
#endif

module GI.GObject.Objects.ParamSpecValueArray
    ( 

-- * Exported types
    ParamSpecValueArray(..)                 ,
    IsParamSpecValueArray                   ,
    toParamSpecValueArray                   ,


 -- * Methods
-- | 
-- 
--  === __Click to display all available methods, including inherited ones__
-- ==== Methods
-- [sink]("GI.GObject.Objects.ParamSpec#g:method:sink"), [stealQdata]("GI.GObject.Objects.ParamSpec#g:method:stealQdata").
-- 
-- ==== Getters
-- [getBlurb]("GI.GObject.Objects.ParamSpec#g:method:getBlurb"), [getDefaultValue]("GI.GObject.Objects.ParamSpec#g:method:getDefaultValue"), [getName]("GI.GObject.Objects.ParamSpec#g:method:getName"), [getNameQuark]("GI.GObject.Objects.ParamSpec#g:method:getNameQuark"), [getNick]("GI.GObject.Objects.ParamSpec#g:method:getNick"), [getQdata]("GI.GObject.Objects.ParamSpec#g:method:getQdata"), [getRedirectTarget]("GI.GObject.Objects.ParamSpec#g:method:getRedirectTarget").
-- 
-- ==== Setters
-- [setQdata]("GI.GObject.Objects.ParamSpec#g:method:setQdata").

#if defined(ENABLE_OVERLOADING)
    ResolveParamSpecValueArrayMethod        ,
#endif



    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.BasicTypes as B.Types
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GArray as B.GArray
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GHashTable as B.GHT
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.GI.Base.Signals as B.Signals
import qualified Control.Monad.IO.Class as MIO
import qualified Data.Coerce as Coerce
import qualified Data.Text as T
import qualified Data.Kind as DK
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL
import qualified GHC.Records as R
import qualified Data.Word as DW
import qualified Data.Int as DI
import qualified System.Posix.Types as SPT
import qualified Foreign.C.Types as FCT

-- Workaround for https://gitlab.haskell.org/ghc/ghc/-/issues/23392
#if MIN_VERSION_base(4,18,0)
import {-# SOURCE #-} qualified GI.GObject.Objects.ParamSpec as GObject.ParamSpec

#else
import {-# SOURCE #-} qualified GI.GObject.Objects.ParamSpec as GObject.ParamSpec

#endif

-- | Memory-managed wrapper type.
newtype ParamSpecValueArray = ParamSpecValueArray (SP.ManagedPtr ParamSpecValueArray)
    deriving (Eq)

instance SP.ManagedPtrNewtype ParamSpecValueArray where
    toManagedPtr (ParamSpecValueArray p) = p

foreign import ccall "haskell_gi_pspec_type_init_ParamSpecValueArray"
    c_haskell_gi_pspec_type_init_ParamSpecValueArray :: IO B.Types.GType

instance B.Types.TypedObject ParamSpecValueArray where
    glibType = c_haskell_gi_pspec_type_init_ParamSpecValueArray

-- | Type class for types which can be safely cast to t'ParamSpecValueArray', for instance with `toParamSpecValueArray`.
class (SP.BoxedPtr o, SP.TypedObject o, O.IsDescendantOf ParamSpecValueArray o) => IsParamSpecValueArray o
instance (SP.BoxedPtr o, SP.TypedObject o, O.IsDescendantOf ParamSpecValueArray o) => IsParamSpecValueArray o

instance O.HasParentTypes ParamSpecValueArray
type instance O.ParentTypes ParamSpecValueArray = '[GObject.ParamSpec.ParamSpec]

-- | Cast to t'ParamSpecValueArray', for types for which this is known to be safe. For general casts, use 'Data.GI.Base.ManagedPtr.castTo'.
toParamSpecValueArray :: (MIO.MonadIO m, IsParamSpecValueArray o) => o -> m ParamSpecValueArray
toParamSpecValueArray = MIO.liftIO . B.ManagedPtr.unsafeCastTo ParamSpecValueArray

--- XXX Missing getter and/or setter, so no GValue instance could be generated.
#if defined(ENABLE_OVERLOADING)
type family ResolveParamSpecValueArrayMethod (t :: Symbol) (o :: DK.Type) :: DK.Type where
    ResolveParamSpecValueArrayMethod "sink" o = GObject.ParamSpec.ParamSpecSinkMethodInfo
    ResolveParamSpecValueArrayMethod "stealQdata" o = GObject.ParamSpec.ParamSpecStealQdataMethodInfo
    ResolveParamSpecValueArrayMethod "getBlurb" o = GObject.ParamSpec.ParamSpecGetBlurbMethodInfo
    ResolveParamSpecValueArrayMethod "getDefaultValue" o = GObject.ParamSpec.ParamSpecGetDefaultValueMethodInfo
    ResolveParamSpecValueArrayMethod "getName" o = GObject.ParamSpec.ParamSpecGetNameMethodInfo
    ResolveParamSpecValueArrayMethod "getNameQuark" o = GObject.ParamSpec.ParamSpecGetNameQuarkMethodInfo
    ResolveParamSpecValueArrayMethod "getNick" o = GObject.ParamSpec.ParamSpecGetNickMethodInfo
    ResolveParamSpecValueArrayMethod "getQdata" o = GObject.ParamSpec.ParamSpecGetQdataMethodInfo
    ResolveParamSpecValueArrayMethod "getRedirectTarget" o = GObject.ParamSpec.ParamSpecGetRedirectTargetMethodInfo
    ResolveParamSpecValueArrayMethod "setQdata" o = GObject.ParamSpec.ParamSpecSetQdataMethodInfo
    ResolveParamSpecValueArrayMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveParamSpecValueArrayMethod t ParamSpecValueArray, O.OverloadedMethod info ParamSpecValueArray p) => OL.IsLabel t (ParamSpecValueArray -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod @info
#else
    fromLabel _ = O.overloadedMethod @info
#endif

#if MIN_VERSION_base(4,13,0)
instance (info ~ ResolveParamSpecValueArrayMethod t ParamSpecValueArray, O.OverloadedMethod info ParamSpecValueArray p, R.HasField t ParamSpecValueArray p) => R.HasField t ParamSpecValueArray p where
    getField = O.overloadedMethod @info

#endif

instance (info ~ ResolveParamSpecValueArrayMethod t ParamSpecValueArray, O.OverloadedMethodInfo info ParamSpecValueArray) => OL.IsLabel t (O.MethodProxy info ParamSpecValueArray) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.MethodProxy
#else
    fromLabel _ = O.MethodProxy
#endif

#endif

-- XXX Wrapping a foreign struct/union with no known destructor or size, leak?
instance BoxedPtr ParamSpecValueArray where
    boxedPtrCopy = return
    boxedPtrFree = \_x -> return ()



